#include <VRS51L3074_SDCC.h>

#include <string.h>
#include <stdio.h>          // only for sprintf in demo code

#define NUMRECORDS	12
#define RECORD_SZ	16

typedef struct {
  unsigned char dat[RECORD_SZ];
} logrecord;

struct ptrstruct {
	unsigned int index;
	unsigned int count;
	unsigned int rdindex;
	unsigned int rdcount;
};
  
//--Init pointer to FRAM base address
__xdata at 0x8000 logrecord record_buffer[NUMRECORDS];
__xdata at 0x9F00 struct ptrstruct bufpos;

logrecord rcrdbfr;
unsigned int rcrdidx,rcrdcnt;
unsigned int rdrcrdidx,rdrcrdcnt;

/* Add a record to the record base */
void putrecord(void)
{
	memcpy((void *)&record_buffer[rcrdidx],(void *) &rcrdbfr,sizeof(logrecord));	// write record
	rcrdcnt++;			// inc record count
	if(++rcrdidx>=NUMRECORDS)	// keep track of circular boundaries for index
		rcrdidx=0;
	if(rcrdcnt >= NUMRECORDS){	// if storage is full, account for overwritten records
		rcrdcnt=NUMRECORDS;		
		if(rdrcrdcnt)
		  rdrcrdcnt--;
		if(!rdrcrdcnt)	
		  rdrcrdidx=rcrdidx;
	}
	// update index and record count in FRAM
	bufpos.index=rcrdidx;	// save to main copy
	bufpos.count=rcrdcnt;
	bufpos.rdindex=rdrcrdidx;	// save to main copy
	bufpos.rdcount=rdrcrdcnt;
}

/* Get (extract) oldest non-read record from database */
int getrecord(void)
{
        if((rcrdcnt-rdrcrdcnt)<=0)
	  return(-1);
	memcpy((void *)&rcrdbfr,(void *)&record_buffer[rdrcrdidx],sizeof(logrecord));
	rdrcrdcnt++;			// inc read record count
	if(rdrcrdcnt >= rcrdcnt) {	// if all read, no inc ptr
		rdrcrdcnt=rcrdcnt;
		rdrcrdidx=rcrdidx;
	}
	else if(++rdrcrdidx>=NUMRECORDS) // keep track of circular boundaries for index
		rdrcrdidx=0;
	// update index and record count in FRAM
	bufpos.rdindex=rdrcrdidx;	// save to main copy
	bufpos.rdcount=rdrcrdcnt;
	return(rcrdcnt-rdrcrdcnt);
}

/* Read a record from database, idx: 0..rcrdcnt-1 */
void readrecord(unsigned int idx)
{
register int j;

	j=rcrdidx-rcrdcnt+idx;
	if(j<0)
		j+=NUMRECORDS;
	else if(j>=NUMRECORDS)
		 j-=NUMRECORDS;

	memcpy((void *)&rcrdbfr,(void *)&record_buffer[j],sizeof(logrecord));
}


 void main (void)
{
  int i;
  
	DEVMEMCFG  |= 0xC0;					// Activate FRAM
/*	FRAMCFG1=1;                              // Disable Write Protect
	while(!(FRAMCFG1&(1<<7)));
	FRAMCFG2=0;
	FRAMCFG1=7;
	while(!(FRAMCFG1&(1<<7)));
	FRAMCFG1=3;
	while(!(FRAMCFG1&(1<<7)));
*/
/*	bufpos.index=0;                                 // First init
	bufpos.count=0;
	bufpos.rdindex=0;             
	bufpos.rdcount=0;
*/
	rcrdidx=bufpos.index;                   // Recover pointers from FRAM
	rcrdcnt=bufpos.count;
	rdrcrdidx=bufpos.rdindex;               // Recover pointers from FRAM
	rdrcrdcnt=bufpos.rdcount;

	for(i=0;i<NUMRECORDS;i++){
	  sprintf((void *)rcrdbfr,"Record #%d",i+1);
	  putrecord();
	}
	for(i=0;i<rcrdcnt;i++)
	  readrecord(i);

	i=0;
	while(getrecord()>0)
	  i++;
}
